/*
 * PmodMAXSONAR.c
 *
 *  Created on: Nov 29, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"

#include "PmodMAXSONAR.h"

// The handle to the UART port used for the communication with the PmodMAXSONAR module.
UART_HandleTypeDef pmodMaxsonarUart;

void PmodMAXSONAR_Config(void)
{
  __HAL_RCC_GPIOF_CLK_ENABLE();

  // Configure the interrupts on PF11 (PWM) pin. The interrupts are active on both edges
  // to show the PWM signal duty cycle.
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Pin = GPIO_PIN_11;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 2, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

  // The UART interface is used only for data reception. The TX pin should be kept high.
  pmodMaxsonarUart.Instance = USART2;
  pmodMaxsonarUart.Init.BaudRate = 9600;
  pmodMaxsonarUart.Init.WordLength = UART_WORDLENGTH_8B;
  pmodMaxsonarUart.Init.StopBits = UART_STOPBITS_1;
  pmodMaxsonarUart.Init.Parity = UART_PARITY_NONE;
  pmodMaxsonarUart.Init.Mode = UART_MODE_RX;
  pmodMaxsonarUart.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  pmodMaxsonarUart.Init.OverSampling = UART_OVERSAMPLING_16;
  pmodMaxsonarUart.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;

  HAL_UART_Init(&pmodMaxsonarUart);
}

void PmodMAXSONAR_HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  // Enable the clocks for GPIO pins used by the USART2 port (PA2, PA3).
  __HAL_RCC_USART2_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF7_USART2;
  GPIO_InitStruct.Pin = GPIO_PIN_3;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  // The TX pin (PA2) - RX of the PmodMAXSONAR should be left in the high state
  // and not used for communication.
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pin = GPIO_PIN_2;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_2, GPIO_PIN_SET);

  // Enable the interrupts generated by USART2 port.
  HAL_NVIC_SetPriority(USART2_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(USART2_IRQn);
}

void PmodMAXSONAR_Read(uint8_t* data, int len)
{
  // Non-blocking data reception.
  HAL_UART_Receive_IT(&pmodMaxsonarUart, data, len);
}
