/*
******************************************************************************
File:     main.c
Info:     Generated by Atollic TrueSTUDIO(R) 8.0.0   2017-10-28

The MIT License (MIT)
Copyright (c) 2009-2017 Atollic AB

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

******************************************************************************
*/

#include "stm32l4xx.h"
#include "stm32l4xx_hal.h"
#include "system_clock.h"
#include "serial.h"
#include "led.h"

#include "PmodMAXSONAR.h"

typedef struct {
  uint8_t buffer[128];
  uint32_t index;
} DataBuffer;

// Data buffer used for data reception.
DataBuffer dataBuffer;

// Data ready flag.
uint8_t dataReady = 0;

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  Led_Config();
  Serial_Config();

  PmodMAXSONAR_Config();

  // Clear the data buffer and start data reception.
  dataBuffer.index = 0;
  PmodMAXSONAR_Read(dataBuffer.buffer, 1);

  while(1)
  {
    while(dataReady == 0)
      __NOP();

    // Send the received data to serial port.
    Serial_Write((char*)dataBuffer.buffer, dataBuffer.index);

    // Clear data and start reception.
    dataReady = 0;
    dataBuffer.index = 0;
    PmodMAXSONAR_Read(dataBuffer.buffer, 1);
  }
}

/**
 * @brief General UART configuration function called by the HAL library during each port
 *        initialization. This function calls the MspInit function of the appropriate driver.
 */
void HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
  if(huart->Instance == USART2)
    PmodMAXSONAR_HAL_UART_MspInit(huart);
  else if(huart->Instance == LPUART1)
    Serial_HAL_UART_MspInit(huart);
}
/**
 * @brief Data reception interrupt handler. The interrupt is configured by the PmodMAXSONAR
 *        driver (USART2).
 */
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
  // Look for the end of the line and notify the main loop.
  // Otherwise receive next character.
  if(dataBuffer.buffer[dataBuffer.index] == '\r') {
    dataBuffer.index++;
    dataReady = 1;
  } else {
    dataBuffer.index++;
    PmodMAXSONAR_Read(&dataBuffer.buffer[dataBuffer.index], 1);
  }
}

/**
 * @brief GPIO interrupt handler. The interrupt is configured by the PmodMAXSONAR driver and shows the state
 *        of the PWM line.
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
  if(HAL_GPIO_ReadPin(GPIOF, GPIO_PIN_11) == GPIO_PIN_SET)
    Led_TurnOn(LED0);
  else
    Led_TurnOff(LED0);
}

